// Copyright(c) 2021 ReliaQube. All rights reserved.
package com.nationrel.modules.alta.service;

import com.alibaba.fastjson.JSONObject;
import com.nationrel.modules.alta.config.AllVariableConfig;
import com.nationrel.modules.alta.constant.*;
import com.nationrel.modules.alta.dto.AltaFastCalculateDto;
import com.nationrel.modules.alta.entity.AltaColumn;
import com.nationrel.modules.alta.entity.AltaData;
import com.nationrel.modules.alta.entity.AltaSetting;
import com.nationrel.modules.alta.enums.AltaCalcParametersEnum;
import com.nationrel.modules.alta.general.domain.enums.reliaqube.ReliaqubeParamDataCheckEnum;
import com.nationrel.modules.alta.general.exception.DescribeException;
import com.nationrel.modules.alta.general.util.common.BasicDataUtil;
import com.nationrel.modules.alta.model.AlcaCalcRequestModel;
import lombok.RequiredArgsConstructor;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.methods.PostMethod;
import org.apache.commons.httpclient.methods.RequestEntity;
import org.apache.commons.httpclient.methods.StringRequestEntity;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.params.CoreConnectionPNames;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.SocketTimeoutException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @version V1.0
 * @ClassName RldAnalysisChartDataServiceImpl
 * @Description api调用
 * @Author aaron
 * @CreateDate 2018-11-1
 */
@SuppressWarnings("AliDeprecation")
@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class AltaRgaApiService {

    private final AllVariableConfig allVariableConfig;
    private int retry;

    public String getResultsCallApi(JSONObject requestData, String url, String contentType, String acceptType) {
        PostMethod postMethod = null;
        String responseResult = "";
        try {
            HttpClient clientPlot = new HttpClient();
            postMethod = new PostMethod(url);
            RequestEntity requestEntityPlot = new StringRequestEntity(
                    requestData.toString(), "text/xml", "UTF-8");
            postMethod.setRequestEntity(requestEntityPlot);
            postMethod.setRequestHeader("Content-Type", contentType);
            if (StringUtils.isNotEmpty(acceptType)) {
                postMethod.setRequestHeader("Accept", acceptType);
            }
            clientPlot.executeMethod(postMethod);
            responseResult = new String(postMethod.getResponseBodyAsString().getBytes("UTF-8"));
        } catch (Exception e) {
            if (retry < 3) {
                retry++;
                // 如果连接失效，重试
                getResultsCallApi(requestData, url, contentType, acceptType);
            } else {
                retry = 0;
                e.printStackTrace();
                //throw new ApplicationRuntimeException("无法链接寿命数据算法服务器");
            }
        } finally {
            postMethod.releaseConnection();
        }
        System.err.println("responseResult => " + responseResult);
        return responseResult;
    }

    public String getResultsCallApi(com.alibaba.fastjson.JSONObject requestData, String url, String contentType, String acceptType, String body) {
        PostMethod postMethod = null;
        String responseResult = "";
        try {
            HttpClient clientPlot = new HttpClient();
            System.err.println(body);
            System.err.println(url);
            postMethod = new PostMethod(url);
            RequestEntity requestEntityPlot = new StringRequestEntity(requestData.toString(), "text/xml", "UTF-8");
            postMethod.setRequestEntity(requestEntityPlot);
            postMethod.setRequestHeader("Content-Type", contentType);
            if (StringUtils.isNotEmpty(acceptType)) {
                postMethod.setRequestHeader("Accept", acceptType);
            }

            postMethod.setRequestBody(AltaConstants.SERVER_SYSTEM_LINUX.equals(allVariableConfig.getServerSystem()) ? body : body.replaceAll("\\\":\\\"\\[", "\":[").replaceAll("]\",\"", "],\""));
            clientPlot.getParams().setParameter(CoreConnectionPNames.CONNECTION_TIMEOUT, 5000);
            clientPlot.getParams().setParameter(CoreConnectionPNames.SO_TIMEOUT, 15000);
            clientPlot.executeMethod(postMethod);
//            responseResult = new String(postMethod.getResponseBodyAsString().getBytes("UTF-8"));
            InputStream inputStream = postMethod.getResponseBodyAsStream();
            BufferedReader br = new BufferedReader(new InputStreamReader(inputStream));
            StringBuffer stringBuffer = new StringBuffer();
            String str = "";
            while ((str = br.readLine()) != null) {
                stringBuffer.append(str);
            }
            responseResult = stringBuffer.toString();
        } catch (SocketTimeoutException e) {
            throw new DescribeException(ReliaqubeParamDataCheckEnum.当前数据在该模型下无法进行分析, 1);
        } catch (Exception e) {
            if (retry < 3) {
                retry++;
                // 如果连接失效，重试
                getResultsCallApi(requestData, url, contentType, acceptType, body);
            } else {
                retry = 0;
                e.printStackTrace();
//				throw new Exception("无法链接服务器");
            }
        } finally {
            postMethod.releaseConnection();
        }
        System.err.println("responseResult => " + responseResult);
        return responseResult;
    }


    /**
     * 根据当前的设置生成需要调用的url
     *
     * @param distributionType
     * @param lifeStressModel
     * @return
     */
    public String getAltaUri(Integer distributionType, Integer lifeStressModel) {
        String uri = null;
        if (distributionType.equals(AltaDistributionTypeEnum.Weibull.getIndex())) {
            if (lifeStressModel.equals(AltaLifeStressModelEnum.Arrhenius.getIndex())
                    || lifeStressModel.equals(AltaLifeStressModelEnum.Eyring.getIndex())
                    || lifeStressModel.equals(AltaLifeStressModelEnum.IPL.getIndex())) {
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.WEIBULL_1_STRESS + "?LSR=" + AltaLifeStressModelEnum.getName(lifeStressModel);
            } else if (lifeStressModel.equals(AltaLifeStressModelEnum.Temp_Humidity.getIndex())
                    || lifeStressModel.equals(AltaLifeStressModelEnum.Temp_NonThermal.getIndex())) {
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.WEIBULL_2_STRESS + "?LSR=" + AltaLifeStressModelEnum.getName(lifeStressModel);
            } else if (lifeStressModel.equals(AltaLifeStressModelEnum.General_Log_Linear.getIndex())) {
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.WEIBULL_MULTI_STRESS;
            }
        } else if (distributionType.equals(AltaDistributionTypeEnum.LogNormal.getIndex())) {
            if (lifeStressModel.equals(AltaLifeStressModelEnum.Arrhenius.getIndex())
                    || lifeStressModel.equals(AltaLifeStressModelEnum.Eyring.getIndex())
                    || lifeStressModel.equals(AltaLifeStressModelEnum.IPL.getIndex())) {
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.LOG_NORMAL_1_STRESS + "?LSR=" + AltaLifeStressModelEnum.getName(lifeStressModel);
            } else if (lifeStressModel.equals(AltaLifeStressModelEnum.Temp_Humidity.getIndex())
                    || lifeStressModel.equals(AltaLifeStressModelEnum.Temp_NonThermal.getIndex())) {
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.LOG_NORMAL_2_STRESS + "?LSR=" + AltaLifeStressModelEnum.getName(lifeStressModel);
            } else if (lifeStressModel.equals(AltaLifeStressModelEnum.General_Log_Linear.getIndex())) {
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.LOG_NORMAL_MULTI_STRESS;
            }
        }
        return uri;

    }

    public String getPlotUri(Integer distributionType, Integer plotType) {
        String uri = null;
        if (plotType.equals(AltaPlotTypeEnum.Reliability.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + AltaPlotTypeEnum.Reliability.getPath() + "?distribution=" + AltaDistributionTypeEnum.getName(distributionType);
        } else if (plotType.equals(AltaPlotTypeEnum.UnReliability.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + AltaPlotTypeEnum.UnReliability.getPath() + "?distribution=" + AltaDistributionTypeEnum.getName(distributionType);
        } else if (plotType.equals(AltaPlotTypeEnum.UseLevelProbability.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + AltaPlotTypeEnum.UseLevelProbability.getPath() + "?distribution=" + AltaDistributionTypeEnum.getName(distributionType);
        } else if (plotType.equals(AltaPlotTypeEnum.FailureRate.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + AltaPlotTypeEnum.FailureRate.getPath() + "?distribution=" + AltaDistributionTypeEnum.getName(distributionType);
        } else if (plotType.equals(AltaPlotTypeEnum.pdf.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + AltaPlotTypeEnum.pdf.getPath() + "?distribution=" + AltaDistributionTypeEnum.getName(distributionType);
        } else if (plotType.equals(AltaPlotTypeEnum.AccelerationFactor.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + AltaPlotTypeEnum.AccelerationFactor.getPath() + "?distribution=" + AltaDistributionTypeEnum.getName(distributionType);
        } else if (plotType.equals(AltaPlotTypeEnum.LifeVsStress.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + AltaPlotTypeEnum.LifeVsStress.getPath() + "?distribution=" + AltaDistributionTypeEnum.getName(distributionType);
        } else if (plotType.equals(AltaPlotTypeEnum.TestLevelProbability.getIndex())) {
            uri = allVariableConfig.getReliaQubeUrl() + AltaPlotTypeEnum.TestLevelProbability.getPath() + "?distribution=" + AltaDistributionTypeEnum.getName(distributionType);
        }
        return uri;
    }

    /**
     * 组装调用接口需要的参数
     *
     * @param altaColumns
     * @param altaDataList
     * @param altaSetting
     * @return
     */
    public Map<String, Object> assemblyData(List<AltaColumn> altaColumns, List<AltaData> altaDataList, AltaSetting altaSetting) {
        String serverSystem = allVariableConfig.getServerSystem();
        List<AlcaCalcRequestModel> dataRowList = new ArrayList<>();
        for (AltaData altaData : altaDataList) {
            AlcaCalcRequestModel dataRow = new AlcaCalcRequestModel();
            dataRow.setTime(altaData.getFailureEndTime());
            dataRow.setLastInspectTime(altaData.getLastInspectTime());
            dataRow.setTimeforRank(0.0);
            if ("1".equals(altaData.getFailureSuspension())) {
                dataRow.setFsStatus(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.failure));
            } else {
                dataRow.setFsStatus(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.suspension));
            }
            dataRow.setNumofForS(altaData.getQuantity());
            dataRow.setFailureSeverity("string");
            dataRow.setFailureMode("string");
            dataRow.setRank(0);
            List orgStressList = new ArrayList();
            for (AltaColumn altaColumn : altaColumns) {
                if (altaColumn.getColumnId().equals(AltaConstants.COLUMN1)) {
                    orgStressList.add(altaData.getComment1());
                }
                if (altaColumn.getColumnId().equals(AltaConstants.COLUMN2)) {
                    orgStressList.add(altaData.getComment2());
                }
                if (altaColumn.getColumnId().equals(AltaConstants.COLUMN3)) {
                    orgStressList.add(altaData.getComment3());
                }
                if (altaColumn.getColumnId().equals(AltaConstants.COLUMN4)) {
                    orgStressList.add(altaData.getComment4());
                }
                if (altaColumn.getColumnId().equals(AltaConstants.COLUMN5)) {
                    orgStressList.add(altaData.getComment5());
                }
                if (altaColumn.getColumnId().equals(AltaConstants.COLUMN6)) {
                    orgStressList.add(altaData.getComment6());
                }
                if (altaColumn.getColumnId().equals(AltaConstants.COLUMN7)) {
                    orgStressList.add(altaData.getComment7());
                }
                if (altaColumn.getColumnId().equals(AltaConstants.COLUMN8)) {
                    orgStressList.add(altaData.getComment8());
                }
            }
            dataRow.setOrgStress(orgStressList);
            dataRow.setStressGroupId(0);
            dataRow.setOrignialId(0);
            dataRowList.add(dataRow);
        }
        Map<String, Object> requestMap = new HashMap<>();
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.dataRow), dataRowList);
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.timeSorted), false);
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.stressSorted), false);
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.rankTimeSorted), false);
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.pointsXYSorted), false);
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.rankTimeCalculated), false);
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.totalNumStress), altaColumns.size());
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.endTime), 0);
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.totalNumFailures), 0);
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.totalNumSuspensions), 0);
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.totalNumFailureandSuspensions), 0);
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.totalFailureRows), 0);
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.totalSuspensionRows), 0);
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.totalNumofRows), 0);
        // 是否包含S
        boolean s = false;
        for (AltaData altaData : altaDataList) {
            if ("0".equals(altaData.getFailureSuspension())) {
                s = true;
                break;
            }
        }

        // A = 右删失
        // B = 左删失
        if (altaSetting.getRightFlag().equals(AltaConstants.FLAG_CHECKED) && altaSetting.getLeftFlag().equals(AltaConstants.FLAG_CHECKED) && s) {
            // 同时选择AB，并且有S
            requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.dataType), AltaDataTypeEnum.IntervalFailureAndSuspension);
        } else if (altaSetting.getRightFlag().equals(AltaConstants.FLAG_CHECKED) && altaSetting.getLeftFlag().equals(AltaConstants.FLAG_CHECKED) && !s) {
            // 同时选择AB，并且没有S
            requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.dataType), AltaDataTypeEnum.IntervalFailureOnly);
        } else if (altaSetting.getRightFlag().equals(AltaConstants.FLAG_NO_CHECKED) && altaSetting.getLeftFlag().equals(AltaConstants.FLAG_NO_CHECKED)) {
            // 都不选择
            requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.dataType), AltaDataTypeEnum.ExactFailureOnly);
        } else if (altaSetting.getRightFlag().equals(AltaConstants.FLAG_CHECKED) && !s) {
            // 选择A且所有的数据均为F时；
            requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.dataType), AltaDataTypeEnum.ExactFailureOnly);
        } else if (altaSetting.getRightFlag().equals(AltaConstants.FLAG_CHECKED) && s) {
            // 选择A且至少有一个数据为S时；
            requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.dataType), AltaDataTypeEnum.ExactFailureAndSuspension);
        } else if (altaSetting.getLeftFlag().equals(AltaConstants.FLAG_CHECKED) && !s) {
            // 选择B且所有的数据均为F时；
            requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.dataType), AltaDataTypeEnum.IntervalFailureOnly);
        } else if (altaSetting.getLeftFlag().equals(AltaConstants.FLAG_CHECKED) && s) {
            // 选择B且至少有一个数据为S时；
            requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.dataType), AltaDataTypeEnum.IntervalFailureAndSuspension);
        }
        // 1，选择右删失且所有的数据均为F时;
//		if (altaSetting.getRightFlag().equals("1") && !s) {
//			requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.DataType), AltaDataTypeEnum.ExactFailureOnly.getName());
//		}
        // 2，选择左删失且所有的数据均为F时;
//		if (altaSetting.getLeftFlag().equals("1") && !s) {
//			requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.DataType), AltaDataTypeEnum.IntervalFailureOnly.getName());
//		}
        // 3，选择右删失且至少有一个数据为S时;
        // 4，选择左删失且至少有一个数据为S时;
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.dataName), "string");
        requestMap.put(AltaCalcParametersEnum.getParameter(serverSystem, AltaCalcParametersEnum.failureRank), new Integer[0]);
        return requestMap;
    }

    /**
     * 获取快速计算的请求路径
     *
     * @param quickCalcType
     * @param input
     * @return
     */
    public String getQuickCalcUri(Integer quickCalcType, AltaFastCalculateDto input) {
        String uri = null;
        switch (quickCalcType) {
            case 0:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.CHAR_LIFE;
                break;
            case 1:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.RELIABILITY + "?time=" + input.getTime();
                break;
            case 2:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.UN_RELIABILITY + "?time=" + input.getTime();
                break;
            case 3:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.COND_RELIABILITY + "?currentTime=" + input.getCurrentTime() + "&missionTime=" + input.getMissionTime();
                break;
            case 4:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.COND_UN_RELIABILITY + "?currentTime=" + input.getCurrentTime() + "&missionTime=" + input.getMissionTime();
                break;
            case 5:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.FAILURE_RATE + "?time=" + input.getTime();
                break;
            case 6:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.TIME_AT_RELIABILITY + "?Rel=" + input.getRel();
                break;
            case 7:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.TIME_AT_UN_RELIABILITY + "?UnRel=" + input.getUnRel();
                break;
            case 8:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.MEAN_LIFE;
                break;
            case 9:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.ACCELERATION_FACTOR;
                break;
            default:
                uri = BasicDataUtil.EMPTY_STR;
        }
        return uri;
    }

    /**
     * 获取快速计算的上下限请求路径
     *
     * @param quickCalcType
     * @param input
     * @return
     */
    public String getQuickCalcBoundsUri(Integer quickCalcType, AltaFastCalculateDto input) {
        String uri = null;
        switch (quickCalcType) {
            case 0:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.CHAR_LIFE;
                break;
            case 1:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.BOUNDS_RELIABILITY + "?time=" + input.getTime() + "&CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 2:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.BOUNDS_UN_RELIABILITY + "?time=" + input.getTime() + "&CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 3:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.BOUNDS_COND_RELIABILITY + "?currentTime=" + input.getCurrentTime() + "&missionTime=" + input.getMissionTime() + "&CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 4:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.BOUNDS_COND_UN_RELIABILITY + "?currentTime=" + input.getCurrentTime() + "&missionTime=" + input.getMissionTime() + "&CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 5:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.BOUNDS_FAILURE_RATE + "?time=" + input.getTime() + "&CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 6:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.BOUNDS_TIME_AT_RELIABILITY + "?R=" + input.getRel() + "&CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 7:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.BOUNDS_TIME_AT_UN_RELIABILITY + "?UR=" + input.getUnRel() + "&CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 8:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.BOUNDS_MEAN_LIFE + "?CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            case 9:
                uri = allVariableConfig.getReliaQubeUrl() + AltaConstants.BOUNDS_ACCELERATION_FACTOR + "?CL=" + input.getCl() + "&sided=" + input.getSided();
                break;
            default:
                uri = BasicDataUtil.EMPTY_STR;
                break;
        }
        return uri;
    }
}
